(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('immer'), require('@datorama/akita')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'immer', '@datorama/akita'], factory) :
  (global = global || self, factory(global.UtilsReactAkita = {}, global.React, global.immer, global.akita));
}(this, (function (exports, react, immer, akita) { 'use strict';

  /**
   *  @description
   *
   * useObservable(): Custom hook for using observables within React view components
   *
   * React view components use state [and props] to render JSX (templates).When the state
   * values will be updated asynchronously via emissions from Observable stream this becomes
   * problematic.
   *
   * Developers must:
   *  - subscribe to the stream
   *  - update the component state with emitted stream values
   *  - unsubscribe from the stream
   *    - when a new observable instance is available
   *    - when the component unmounts
   *  - trigger view component re-renders
   *
   * This 'typed' custom hook simplifies the entire process ^ and internally manages the subscription lifecycles.
   * The hook also dramatically reduces the code complexity WITHIN a React component
   * > For Angular developers, this hook provides the same functionality as the template `async` pipe construct.
   *
   * `useObservable<T>(source:Observable<T>,initialVal:T): [val:T, setObservable]`
   *
   * @code
   *   const [people, setPeople$]     = useObservable<Contact[]>(null, []);
   *   const [criteria, setCriteria$] = useObservable<string>(null, '');
   *
   *   useIonViewWillEnter(() => {
   *     const term$ = emitter.asObservable();
   *     const allContacts$ = facade.contacts$.pipe(takeUntil(term$));
   *     const searchTerm$ = facade.autoSearch(term$);
   *
   *     setCriteria$(term$);
   *     setPeople$(merge(allContacts$, searchTerm$));
   *   });
   *
   *   return (
   *     <IonList>
   *      {
   *        people.map((person, idx) => <ContactListItem key={idx} person={person} />
   *      }
   *      </IonList>
   *   );
   *
   * @publicApi
   */
  /**
   * Manage Observable subscµription and report value emission.
   * - support dynamic reset of stream source
   * - support auto-unsubscribe when view component unmounts
   * @param observable$ Stream source
   * @param initialValue initial value; to simulate 1st stream emission
   */

  function useObservable(observable$, initialValue) {
    var _a = react.useState(observable$),
        source$ = _a[0],
        setObservable = _a[1];

    var _b = react.useState(initialValue),
        value = _b[0],
        setValue = _b[1];

    var reportError = function reportError(err) {
      setValue(undefined);
      console.error("useObservable() error: " + JSON.stringify(err));
    };

    react.useEffect(function () {
      if (source$) {
        var s_1 = source$.subscribe(setValue, reportError);
        return function () {
          s_1.unsubscribe();
        };
      }
    }, [source$]);
    return [value, setObservable];
  }

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */
  /* global Reflect, Promise */

  var extendStatics = function(d, b) {
      extendStatics = Object.setPrototypeOf ||
          ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
          function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
      return extendStatics(d, b);
  };

  function __extends(d, b) {
      extendStatics(d, b);
      function __() { this.constructor = d; }
      d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  }

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  function isFunction(x) {
      return typeof x === 'function';
  }

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  var _enable_super_gross_mode_that_will_cause_bad_things = false;
  var config = {
      Promise: undefined,
      set useDeprecatedSynchronousErrorHandling(value) {
          if (value) {
              var error = /*@__PURE__*/ new Error();
              /*@__PURE__*/ console.warn('DEPRECATED! RxJS was set to use deprecated synchronous error handling behavior by code at: \n' + error.stack);
          }
          _enable_super_gross_mode_that_will_cause_bad_things = value;
      },
      get useDeprecatedSynchronousErrorHandling() {
          return _enable_super_gross_mode_that_will_cause_bad_things;
      },
  };

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  function hostReportError(err) {
      setTimeout(function () { throw err; }, 0);
  }

  /** PURE_IMPORTS_START _config,_util_hostReportError PURE_IMPORTS_END */
  var empty = {
      closed: true,
      next: function (value) { },
      error: function (err) {
          if (config.useDeprecatedSynchronousErrorHandling) {
              throw err;
          }
          else {
              hostReportError(err);
          }
      },
      complete: function () { }
  };

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  var isArray = /*@__PURE__*/ (function () { return Array.isArray || (function (x) { return x && typeof x.length === 'number'; }); })();

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  function isObject(x) {
      return x !== null && typeof x === 'object';
  }

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  var UnsubscriptionErrorImpl = /*@__PURE__*/ (function () {
      function UnsubscriptionErrorImpl(errors) {
          Error.call(this);
          this.message = errors ?
              errors.length + " errors occurred during unsubscription:\n" + errors.map(function (err, i) { return i + 1 + ") " + err.toString(); }).join('\n  ') : '';
          this.name = 'UnsubscriptionError';
          this.errors = errors;
          return this;
      }
      UnsubscriptionErrorImpl.prototype = /*@__PURE__*/ Object.create(Error.prototype);
      return UnsubscriptionErrorImpl;
  })();
  var UnsubscriptionError = UnsubscriptionErrorImpl;

  /** PURE_IMPORTS_START _util_isArray,_util_isObject,_util_isFunction,_util_UnsubscriptionError PURE_IMPORTS_END */
  var Subscription = /*@__PURE__*/ (function () {
      function Subscription(unsubscribe) {
          this.closed = false;
          this._parentOrParents = null;
          this._subscriptions = null;
          if (unsubscribe) {
              this._unsubscribe = unsubscribe;
          }
      }
      Subscription.prototype.unsubscribe = function () {
          var errors;
          if (this.closed) {
              return;
          }
          var _a = this, _parentOrParents = _a._parentOrParents, _unsubscribe = _a._unsubscribe, _subscriptions = _a._subscriptions;
          this.closed = true;
          this._parentOrParents = null;
          this._subscriptions = null;
          if (_parentOrParents instanceof Subscription) {
              _parentOrParents.remove(this);
          }
          else if (_parentOrParents !== null) {
              for (var index = 0; index < _parentOrParents.length; ++index) {
                  var parent_1 = _parentOrParents[index];
                  parent_1.remove(this);
              }
          }
          if (isFunction(_unsubscribe)) {
              try {
                  _unsubscribe.call(this);
              }
              catch (e) {
                  errors = e instanceof UnsubscriptionError ? flattenUnsubscriptionErrors(e.errors) : [e];
              }
          }
          if (isArray(_subscriptions)) {
              var index = -1;
              var len = _subscriptions.length;
              while (++index < len) {
                  var sub = _subscriptions[index];
                  if (isObject(sub)) {
                      try {
                          sub.unsubscribe();
                      }
                      catch (e) {
                          errors = errors || [];
                          if (e instanceof UnsubscriptionError) {
                              errors = errors.concat(flattenUnsubscriptionErrors(e.errors));
                          }
                          else {
                              errors.push(e);
                          }
                      }
                  }
              }
          }
          if (errors) {
              throw new UnsubscriptionError(errors);
          }
      };
      Subscription.prototype.add = function (teardown) {
          var subscription = teardown;
          if (!teardown) {
              return Subscription.EMPTY;
          }
          switch (typeof teardown) {
              case 'function':
                  subscription = new Subscription(teardown);
              case 'object':
                  if (subscription === this || subscription.closed || typeof subscription.unsubscribe !== 'function') {
                      return subscription;
                  }
                  else if (this.closed) {
                      subscription.unsubscribe();
                      return subscription;
                  }
                  else if (!(subscription instanceof Subscription)) {
                      var tmp = subscription;
                      subscription = new Subscription();
                      subscription._subscriptions = [tmp];
                  }
                  break;
              default: {
                  throw new Error('unrecognized teardown ' + teardown + ' added to Subscription.');
              }
          }
          var _parentOrParents = subscription._parentOrParents;
          if (_parentOrParents === null) {
              subscription._parentOrParents = this;
          }
          else if (_parentOrParents instanceof Subscription) {
              if (_parentOrParents === this) {
                  return subscription;
              }
              subscription._parentOrParents = [_parentOrParents, this];
          }
          else if (_parentOrParents.indexOf(this) === -1) {
              _parentOrParents.push(this);
          }
          else {
              return subscription;
          }
          var subscriptions = this._subscriptions;
          if (subscriptions === null) {
              this._subscriptions = [subscription];
          }
          else {
              subscriptions.push(subscription);
          }
          return subscription;
      };
      Subscription.prototype.remove = function (subscription) {
          var subscriptions = this._subscriptions;
          if (subscriptions) {
              var subscriptionIndex = subscriptions.indexOf(subscription);
              if (subscriptionIndex !== -1) {
                  subscriptions.splice(subscriptionIndex, 1);
              }
          }
      };
      Subscription.EMPTY = (function (empty) {
          empty.closed = true;
          return empty;
      }(new Subscription()));
      return Subscription;
  }());
  function flattenUnsubscriptionErrors(errors) {
      return errors.reduce(function (errs, err) { return errs.concat((err instanceof UnsubscriptionError) ? err.errors : err); }, []);
  }

  /** PURE_IMPORTS_START  PURE_IMPORTS_END */
  var rxSubscriber = /*@__PURE__*/ (function () {
      return typeof Symbol === 'function'
          ? /*@__PURE__*/ Symbol('rxSubscriber')
          : '@@rxSubscriber_' + /*@__PURE__*/ Math.random();
  })();

  /** PURE_IMPORTS_START tslib,_util_isFunction,_Observer,_Subscription,_internal_symbol_rxSubscriber,_config,_util_hostReportError PURE_IMPORTS_END */
  var Subscriber = /*@__PURE__*/ (function (_super) {
      __extends(Subscriber, _super);
      function Subscriber(destinationOrNext, error, complete) {
          var _this = _super.call(this) || this;
          _this.syncErrorValue = null;
          _this.syncErrorThrown = false;
          _this.syncErrorThrowable = false;
          _this.isStopped = false;
          switch (arguments.length) {
              case 0:
                  _this.destination = empty;
                  break;
              case 1:
                  if (!destinationOrNext) {
                      _this.destination = empty;
                      break;
                  }
                  if (typeof destinationOrNext === 'object') {
                      if (destinationOrNext instanceof Subscriber) {
                          _this.syncErrorThrowable = destinationOrNext.syncErrorThrowable;
                          _this.destination = destinationOrNext;
                          destinationOrNext.add(_this);
                      }
                      else {
                          _this.syncErrorThrowable = true;
                          _this.destination = new SafeSubscriber(_this, destinationOrNext);
                      }
                      break;
                  }
              default:
                  _this.syncErrorThrowable = true;
                  _this.destination = new SafeSubscriber(_this, destinationOrNext, error, complete);
                  break;
          }
          return _this;
      }
      Subscriber.prototype[rxSubscriber] = function () { return this; };
      Subscriber.create = function (next, error, complete) {
          var subscriber = new Subscriber(next, error, complete);
          subscriber.syncErrorThrowable = false;
          return subscriber;
      };
      Subscriber.prototype.next = function (value) {
          if (!this.isStopped) {
              this._next(value);
          }
      };
      Subscriber.prototype.error = function (err) {
          if (!this.isStopped) {
              this.isStopped = true;
              this._error(err);
          }
      };
      Subscriber.prototype.complete = function () {
          if (!this.isStopped) {
              this.isStopped = true;
              this._complete();
          }
      };
      Subscriber.prototype.unsubscribe = function () {
          if (this.closed) {
              return;
          }
          this.isStopped = true;
          _super.prototype.unsubscribe.call(this);
      };
      Subscriber.prototype._next = function (value) {
          this.destination.next(value);
      };
      Subscriber.prototype._error = function (err) {
          this.destination.error(err);
          this.unsubscribe();
      };
      Subscriber.prototype._complete = function () {
          this.destination.complete();
          this.unsubscribe();
      };
      Subscriber.prototype._unsubscribeAndRecycle = function () {
          var _parentOrParents = this._parentOrParents;
          this._parentOrParents = null;
          this.unsubscribe();
          this.closed = false;
          this.isStopped = false;
          this._parentOrParents = _parentOrParents;
          return this;
      };
      return Subscriber;
  }(Subscription));
  var SafeSubscriber = /*@__PURE__*/ (function (_super) {
      __extends(SafeSubscriber, _super);
      function SafeSubscriber(_parentSubscriber, observerOrNext, error, complete) {
          var _this = _super.call(this) || this;
          _this._parentSubscriber = _parentSubscriber;
          var next;
          var context = _this;
          if (isFunction(observerOrNext)) {
              next = observerOrNext;
          }
          else if (observerOrNext) {
              next = observerOrNext.next;
              error = observerOrNext.error;
              complete = observerOrNext.complete;
              if (observerOrNext !== empty) {
                  context = Object.create(observerOrNext);
                  if (isFunction(context.unsubscribe)) {
                      _this.add(context.unsubscribe.bind(context));
                  }
                  context.unsubscribe = _this.unsubscribe.bind(_this);
              }
          }
          _this._context = context;
          _this._next = next;
          _this._error = error;
          _this._complete = complete;
          return _this;
      }
      SafeSubscriber.prototype.next = function (value) {
          if (!this.isStopped && this._next) {
              var _parentSubscriber = this._parentSubscriber;
              if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                  this.__tryOrUnsub(this._next, value);
              }
              else if (this.__tryOrSetError(_parentSubscriber, this._next, value)) {
                  this.unsubscribe();
              }
          }
      };
      SafeSubscriber.prototype.error = function (err) {
          if (!this.isStopped) {
              var _parentSubscriber = this._parentSubscriber;
              var useDeprecatedSynchronousErrorHandling = config.useDeprecatedSynchronousErrorHandling;
              if (this._error) {
                  if (!useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                      this.__tryOrUnsub(this._error, err);
                      this.unsubscribe();
                  }
                  else {
                      this.__tryOrSetError(_parentSubscriber, this._error, err);
                      this.unsubscribe();
                  }
              }
              else if (!_parentSubscriber.syncErrorThrowable) {
                  this.unsubscribe();
                  if (useDeprecatedSynchronousErrorHandling) {
                      throw err;
                  }
                  hostReportError(err);
              }
              else {
                  if (useDeprecatedSynchronousErrorHandling) {
                      _parentSubscriber.syncErrorValue = err;
                      _parentSubscriber.syncErrorThrown = true;
                  }
                  else {
                      hostReportError(err);
                  }
                  this.unsubscribe();
              }
          }
      };
      SafeSubscriber.prototype.complete = function () {
          var _this = this;
          if (!this.isStopped) {
              var _parentSubscriber = this._parentSubscriber;
              if (this._complete) {
                  var wrappedComplete = function () { return _this._complete.call(_this._context); };
                  if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                      this.__tryOrUnsub(wrappedComplete);
                      this.unsubscribe();
                  }
                  else {
                      this.__tryOrSetError(_parentSubscriber, wrappedComplete);
                      this.unsubscribe();
                  }
              }
              else {
                  this.unsubscribe();
              }
          }
      };
      SafeSubscriber.prototype.__tryOrUnsub = function (fn, value) {
          try {
              fn.call(this._context, value);
          }
          catch (err) {
              this.unsubscribe();
              if (config.useDeprecatedSynchronousErrorHandling) {
                  throw err;
              }
              else {
                  hostReportError(err);
              }
          }
      };
      SafeSubscriber.prototype.__tryOrSetError = function (parent, fn, value) {
          if (!config.useDeprecatedSynchronousErrorHandling) {
              throw new Error('bad call');
          }
          try {
              fn.call(this._context, value);
          }
          catch (err) {
              if (config.useDeprecatedSynchronousErrorHandling) {
                  parent.syncErrorValue = err;
                  parent.syncErrorThrown = true;
                  return true;
              }
              else {
                  hostReportError(err);
                  return true;
              }
          }
          return false;
      };
      SafeSubscriber.prototype._unsubscribe = function () {
          var _parentSubscriber = this._parentSubscriber;
          this._context = null;
          this._parentSubscriber = null;
          _parentSubscriber.unsubscribe();
      };
      return SafeSubscriber;
  }(Subscriber));

  /** PURE_IMPORTS_START tslib,_Subscriber PURE_IMPORTS_END */
  function map(project, thisArg) {
      return function mapOperation(source) {
          if (typeof project !== 'function') {
              throw new TypeError('argument is not a function. Are you looking for `mapTo()`?');
          }
          return source.lift(new MapOperator(project, thisArg));
      };
  }
  var MapOperator = /*@__PURE__*/ (function () {
      function MapOperator(project, thisArg) {
          this.project = project;
          this.thisArg = thisArg;
      }
      MapOperator.prototype.call = function (subscriber, source) {
          return source.subscribe(new MapSubscriber(subscriber, this.project, this.thisArg));
      };
      return MapOperator;
  }());
  var MapSubscriber = /*@__PURE__*/ (function (_super) {
      __extends(MapSubscriber, _super);
      function MapSubscriber(destination, project, thisArg) {
          var _this = _super.call(this, destination) || this;
          _this.project = project;
          _this.count = 0;
          _this.thisArg = thisArg || _this;
          return _this;
      }
      MapSubscriber.prototype._next = function (value) {
          var result;
          try {
              result = this.project.call(this.thisArg, value, this.count++);
          }
          catch (err) {
              this.destination.error(err);
              return;
          }
          this.destination.next(result);
      };
      return MapSubscriber;
  }(Subscriber));

  var useIsoLayoutEffect = typeof window === 'undefined' ? react.useEffect : react.useLayoutEffect;

  var identity = function identity(s) {
    return s;
  };
  /**
   * Create a store with specified state
   *
   * The store instances is actually a React hook with Store API added
   * All stores are decorated with Status state (isLoading, error, setIsLoading, and setError)
   *
   * @returns useStore: Hook that will emit either all state or partial state [based on passed selector].
   */


  function createStore(createState, options) {
    if (options === void 0) {
      options = {};
    }
    /**
     * Internal, startup queue for computed and watched properties
     *
     * NOTE: is currently only used during store startup configuration
     */


    var computed = {};
    /**
     * Internal Akita Store + Query instances
     * Note: Immer immutability is auto-applied via the `producerFn` configuration
     */

    var name = options.storeName || "ReactAkitStore" + Math.random();
    var store = new akita.Store({}, {
      producerFn: immer.produce,
      name: name
    });
    var query = new akita.Query(store);
    /**
     * setIsLoading() + setError()
     * Status API methods available on 'api'
     */

    var setIsLoading = function setIsLoading(isLoading) {
      if (isLoading === void 0) {
        isLoading = true;
      }

      return store.update(function (s) {
        return __assign(__assign({}, s), {
          isLoading: isLoading
        });
      });
    };

    var setError = function setError(error) {
      store.update(function (s) {
        return __assign(__assign({}, s), {
          error: error
        });
      });
    };
    /**
     * setState() + getState()
     * Build API methods used that delegate to the store and query
     */


    var getState = store.getValue.bind(store);

    var setState = function setState(partial, replace) {
      var isCallback = partial instanceof Function;

      var updateWithValue = function updateWithValue(s) {
        return replace ? s : __assign(__assign({}, s), partial);
      };

      store.update(!isCallback ? updateWithValue : partial);
    };

    var applyTransaction = function applyTransaction(action, options) {
      var msg = function msg(phase) {
        return "---- " + phase + " applyTransaction() ----- ";
      };

      var log = function log(phase) {
        return (options === null || options === void 0 ? void 0 : options.enableLog) && console.log(msg(phase));
      };

      log('start');
      var response = akita.applyTransaction(action, options === null || options === void 0 ? void 0 : options.thisArg);
      log('stop');
      return response;
    };
    /**
     * subscribe()
     * The subscribe function allows components to bind to a state-portion without forcing re-render on changes
     */


    var subscribe = function subscribe(listener, selector) {
      var onNext = function onNext(s) {
        return listener(s);
      };

      var watcher = query.select(selector || identity).subscribe(onNext);
      return function () {
        return watcher.unsubscribe();
      };
    };
    /**
     * addComputedProperty()
     *
     * Inject the computed property into the target 'state':
     *
     * - Subscribe to state changes with the computed property selectors
     * - Use the computed property predicate function to map to a 'value' response
     * - update the targeted computed property with the 'value' response
     *
     * This method is used inside createStore() to configure watches during state
     * initialization.
     *
     * NOTE: since the store/query are not yet ready, we 'queue' these requests
     *       and later in `registerComputedProperties()` we finish the setup
     *       for all computed properties
     */


    var addComputedProperty = function addComputedProperty(property, target) {
      var deferredSetup = function deferredSetup() {
        var makeQuery = function makeQuery(predicate) {
          return query.select(predicate);
        };

        var emitters = property.selectors.map(makeQuery);
        var source$ = akita.combineQueries(emitters).pipe(map(property.predicate));
        source$.subscribe(function (computedValue) {
          callAsync(function (value) {
            setState(function (s) {
              var _a;

              return __assign(__assign({}, s), (_a = {}, _a[property.name] = value, _a));
            });
          }, computedValue);
        });
      };

      computed[property.name] = deferredSetup;
      return target;
    };
    /**
     * Watch for changes in a specific property and then notify 'listener'
     * This method is used inside createStore() to configure watches during state
     * initialization.
     */


    var watchProperty = function watchProperty(property, listener) {
      var deferredSetup = function deferredSetup() {
        var onNext = function onNext(s) {
          return callAsync(listener, s);
        };

        var selector = function selector(s) {
          return s[property];
        };

        var watcher = query.select(selector).subscribe(onNext);
        return function () {
          return watcher.unsubscribe();
        };
      };

      computed[property] = deferredSetup;
    };
    /**
     * Activate all 'queued' computed/watched properties
     */


    var registerComputedProperties = function registerComputedProperties() {
      Object.keys(computed).map(function (propertyName) {
        var registerNow = computed[propertyName];
        registerNow();
        delete computed[propertyName];
      });
    };
    /**
     * Complete streams and close the store
     */


    var destroy = function destroy() {
      return store.destroy();
    };
    /**
     * Create the Store instance with desired API
     */


    var storeAPI = {
      get: getState,
      set: setState,
      applyTransaction: applyTransaction,
      addComputedProperty: addComputedProperty,
      watchProperty: watchProperty,
      observe: subscribe,
      destroy: destroy,
      setIsLoading: setIsLoading,
      setError: setError
    };
    /**
     * Internal utility methods for selectors
     *
     * toObservable(): Build an RxJS stream for the specified selector(s)
     * toStateSlice(): Gather current state values for the specified selector(s)
     */

    var toObservable = function toObservable(selector) {
      if (selector === void 0) {
        selector = identity;
      }

      var list = selector instanceof Array ? selector : [selector];
      return list.length == 1 ? query.select(list[0]) : akita.combineQueries(list.map(function (it) {
        return query.select(it);
      }));
    };

    var getSliceValueFor = function getSliceValueFor(selector) {
      if (selector === void 0) {
        selector = identity;
      }

      var list = selector instanceof Array ? selector : null;

      var getCurrent = function getCurrent(it) {
        return it(getState());
      };

      return list ? list.map(getCurrent) : getCurrent(selector);
    };
    /**
     * userStore()
     *
     * After the store has been created and initialized with api + state, this hoook
     * is created. Developer will use this custom react hook to 'select' state data.
     * The selected data sourcds will re-emit data after changes.
     *
     * This hook is implicitly connected to its associated [parent] store
     *
     * Warning!!
     *   1) Do not use same 'useStore' hook multiple times in the same component.
     *   2) To extract multiple slices, use
     *      a) a single selector to return a reponse (value, hashmap, or tuple)
     *      b) an array of selectors [optimized approach] to return response
     */


    var useStore = function useStore(selector) {
      var initial = react.useState(function () {
        return getSliceValueFor(selector);
      })[0];

      var _a = useObservable(null, initial),
          slice = _a[0],
          setSlice$ = _a[1];

      useIsoLayoutEffect(function () {
        setSlice$(toObservable(selector));
        return function () {
          return destroy();
        };
      }, [selector]);
      return slice;
    };
    /**
     * Initialization of state management
     * Create the immutable state using the 'createState()' callback factory
     *
     * - Reinitialize the internal store,
     * - Register computed and watched properties
     * - inject storeAPI
     */


    var onInit = function onInit() {
      var state = immer.produce({}, function () {
        return __assign({
          error: null,
          isLoading: false
        }, createState(storeAPI));
      });
      reinitStore(store, name, state);
      registerComputedProperties(); // Decorate hook function with API methods, public custom hook/store

      return Object.assign(useStore, storeAPI);
    };

    return onInit();
  } // *******************************************************
  // Hacks
  // *******************************************************

  /**
   * Currently Akita expects a store class to be decorated with @StoreConfi() to set a store name
   * If this is not done, the constructor will assert a `console.error()`
   * @param store
   * @param name
   */

  function reinitStore(store, name, state) {
    if (!!name && store.storeName != name) {
      store.constructor['akitaConfig'].storeName = name;
      store['options'].storeName = name;
    } // Reinitalize


    store['onInit'](state);
  }
  /**
   * Invoke the callback at the end of the microTask
   */


  function callAsync(callback, resolveWith) {
    Promise.resolve(resolveWith).then(callback);
  }

  exports.createStore = createStore;
  exports.useObservable = useObservable;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
